/*
ͻ
 NOM       : STARINTF.C                                                      
 FONCTION  : Procdures d'interfaage du pilote de voies sries RS232C       
           : STARCOMM.EXE au langage C, qu'il soit charg en tant que pilote 
           : "BIOS" ou en tant que DEVICE DRIVER du DOS.                     
 VERSION   : v2.0 (premire dition).                                        
 LANGAGE   : TURBO C version 1.0 et suivantes.                               
 STRUCTURE : Bilbliothque standard de procdures.                           
 REMARQUES : Toutes les fonctions I retournent le code UN_CHECKED si elles   
           : sont appeles alors que "Check_STARCOMM_Present" n'a pas d'abord
           : t appele.                                                    
           : Toutes les fonctions II retournent le code UN_OPENED si elles   
           : sont appeles alors que "Open_COMM" n'a pas d'abord t appel. 
           : Pour utiliser cette bibliothque, il faut donc appeler:         
           : * au dbut du programme:                                        
           :   - "Check_STARCOMM_Present" pour vrifier la prsence du pilote
           :     en mmoire vive,                                            
           :   - "Open_COMM" si le pilote est charg en tant que DEVICE      
           :     DRIVER et que l'on veut utiliser cette interface systme,   
           :   - "Open_Port" pour s'assurer de l'activation des voies sries.
           : *  la fin du programme: (optionnel)                            
           :   - "Close_Port" pour mettre  0 les signaux de contrle du     
           :     modem (permet de ne plus alimenter le modem...) en fermants 
           :     les ports srie.                                            
           :   - "Close_COMM" si le pilote est charg en tant que DEVICE DRV.
 COPYRIGHT : HETRU Fabrice 1991-1994.                                        
ͼ
*/


#define TRUE         1
#define FALSE        0
#define UN_CHECKED   20
#define UN_OPENED    21


#define byte         unsigned char
#define word         unsigned int


#include <mem.h>
/* Accs aux registres CPU */
#include <dos.h>
union  REGS  inregs,outregs;
struct SREGS segregs;
/* Variables globales gnrales. */
static byte RS_accessible = FALSE;     /* Indique si le pilote est charg.   */
static byte ComExist[4]; /* Repre les ports existants suite  "Port_Opened" */
static word CommPort = 0;      /* Port srie concern: 0 (COM1:)  3 (COM4:) */
static word Format;
static byte Voie_active, Pret, Clear_to_send, Sonnerie, Porteuse,
            dDSR, dCTS, dRI, dDCD;
static byte Buff_overflow, Engorgement, Parite, Stop_bit, Break_it;
static unsigned BasePort1 = 0x03F8;
static unsigned BasePort2 = 0x02F8;
static unsigned BasePort3 = 0x03E8;
static unsigned BasePort4 = 0x02E8;
static byte RTS_CTS  = 1;
static byte DTR_DSR  = 2;
static byte BothCmde = 3;
static byte Inactif   =  0;
static byte Local     = 76; /*'L'*/
static byte Eloigne   = 69; /*'E'*/
static byte Bilateral = 66; /*'B'*/
static byte Xon  = 0x11;
static byte Xoff = 0x13;
static byte NonActif = 0; /* Smode0: Les octets dfectueux sont bufferiss */
                          /* comme ils sont dcods...                     */
static byte Remplace = 1; /* Smode1: Octets HS remplacs par code spcial. */
static byte RempNULL = 2; /* Smode2: Octets HS perdus. Le compteur du nb.  */
                          /* d'octets perdus est incrment.               */
/* Variables pour STARCOMM charg en DEVICE DRIVER. */
static char NomDevice[9] = {'C','O','M','M',' ',' ',' ',' ','\0'};
static word handler_voie_serie;
static char port_ouvert = FALSE;


/* ==================== PROCEDURES BASEES SUR L'INT. 14h =================== */
byte Version(char *NumVer)
/* Retourne le numro de version (en ASCII) du pilote trouv en RAM.         */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 6;
    int86(0x14,&inregs,&outregs);
    NumVer[0] = outregs.h.al;
    NumVer[1] = '.';
    outregs.x.ax = outregs.x.di;
    NumVer[2] = outregs.h.ah;
    NumVer[3] = outregs.h.al;
    NumVer[4] = 0;
    return(TRUE);
    }
  else
    {
    NumVer[0] = 0;
    return(UN_CHECKED);
    }
  }


byte Open_Port()
/* Ouvre un port srie. Met  1 les signaux DTR et OUT2 et annule tous les   */
/* codes de comptabilisation d'erreurs en transmissions.                     */
/* Retourne un code 5,8 ou 9 si le port n'a put tre ouvert.                 */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 7;
    inregs.h.al = 0;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte Close_Port()
/* Ferme un port srie. Met  0 les signaux ctrl-modem.                      */
/* Retourne le code 5 ou 7 si le port n'a put tre ferm (car buffer d'ms.  */
/* NON vide OU port inexistant !). 10 indique l'impossibilit de dsallouer  */
/* le buffer du port  fermer (mais le port est ferm physiquement).         */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 7;
    inregs.h.al = 1;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte Port_Opened()
/* Indique si un port srie est ouvert (TRUE) ou ferm.                      */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 7;
    inregs.h.al = 2;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    if (CommPort<4)
      {
      if (outregs.h.ah==5) ComExist[CommPort]=FALSE;
        else ComExist[CommPort]=TRUE;
      }
    return(outregs.h.al);
    }
  else return(UN_CHECKED);
  }


byte Interdit_Port()
/* Interdit l'accs au port COMi: point par "CommPort".                     */
/* Retourne le code d'erreur 10 si une erreur de dsallocation des buffers   */
/* s'est produite  la fermeture du port. L'accs au port sera cependant     */
/* effectivement interdit (erreur ou non...).                                */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x0A;
    inregs.h.al = 0;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte Autorise_Port()
/* Autorise l'accs au port COMi: point par "CommPort" si possible.         */
/* Retourne le code d'erreur 5 si le port n'a put tre ouvert par dfaut     */
/* d'UART trouv. Le port reste alors interdit d'accs.                      */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x0A;
    inregs.h.al = 1;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte SLOW_Ems()
/* Initialisation d'un port srie en mode d'mission lente, sans en modifier */
/* l'tat d'activit.                                                        */
/* Retourne le code 5 si le port n'existe pas.                               */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 7;
    inregs.h.al = 3;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte FAST_Ems()
/* Initialisation d'un port srie en mode d'mission rapide, sans en modifier*/
/* l'tat d'activit.                                                        */
/* Retourne le code 5 si le port n'existe pas.                               */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 7;
    inregs.h.al = 4;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte Send_SLOW()
/* Indique si un port srie est en mode d'mission lente (sous IT d'horloge).*/
  {
  if (RS_accessible)
    {
    inregs.h.ah = 7;
    inregs.h.al = 5;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.al);
    }
  else return(UN_CHECKED);
  }


void Get_Ports_Adresses()
/* Lit les adresses de base des UART dans la boite aux lettres du BIOS.      */
  {
  movedata(0x40,0x00,FP_SEG(&BasePort1),FP_OFF(&BasePort1),8);
  }


byte Infos_Uart(byte *Uart_type, word *Uart_adresse, byte *Irq,
     byte *FIFO_actif, byte *Taille_FIFO)
/* Retour d'informations sur les UART utiliss.                              */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 9;
    inregs.h.al = 0;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    *Uart_type = outregs.h.al;
    *Uart_adresse = outregs.x.bx;
    *Irq = outregs.h.ch;
    if (outregs.h.cl==0)
      {
      *FIFO_actif = FALSE;
      *Taille_FIFO = 1;
      }
    else
      {
      *FIFO_actif = TRUE;
      *Taille_FIFO = outregs.h.cl;
      }
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte Set_New_Uart(word Uart_adresse,byte Irq,byte FIFO_actif,byte Taille_FIFO)
/* Dfinition de nouveaux paramtres physiques pour un port srie.           */
/* Retourne 0 si Ok, 5: port inexistant, 11: port ouvert !, 12: Adresse INOP.*/
  {
  if (RS_accessible)
    {
    inregs.h.ah = 9;
    inregs.h.al = 1;
    inregs.x.dx = CommPort;
    inregs.x.bx = Uart_adresse;
    inregs.h.ch = Irq;
    if (FIFO_actif) inregs.h.cl = Taille_FIFO;
      else inregs.h.cl = 0;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte Infos_Buffs(byte *Nb_actifs, byte *Nb_maxi, word *Taille_InBuff,
     word *Taille_OutBuff)
/* Retour d'informations sur les buffers utilisables par STARCOMM.           */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x0D;
    int86(0x14,&inregs,&outregs);
    *Taille_InBuff = outregs.x.bx;
    *Taille_OutBuff = outregs.x.cx;
    *Nb_actifs = outregs.h.dh;
    *Nb_maxi = outregs.h.dl;
    return(FALSE);
    }
  else return(UN_CHECKED);
  }


byte Init_Port(byte longueur, byte stop_bit, byte parity, long word vitesse,
     word *Taille_InBuff, word *Taille_OutBuff)
/* Fonction FACULTATIVE: 9600 Bds, no parity, 1 stop, 8 data bit par dfaut  */
/* La voie active est necessairement celle gre par le pilote actif...      */
/* Paramtres:                                                               */
/*     longueur = '5', '6', '7', ou '8' bits,                                */
/*     stop_bit = '1' ou '2',                                                */
/*     parity   = <'P'|'p'>aire, <'I'|'i'>mpaire, <'N'|'n'>o parity,         */
/*                <'T'|'t'>ravail, <'R'|'r'>epos,                            */
/*     vitesse  = {110,150,300,600,1200,2400,4800,9600                       */
/*                19200,28800,38400,57600,115200} bauds.                     */
/* Retourne UN_CHECKED si la prsence du pilote en RAM non vrifie. Sinon,  */
/* retourne 1 si paramtres incorrectes (erreur=TRUE), 0 (FALSE) si OK.      */
/* Indique les tailles des buffers utiliss dans tous les cas en retour.     */
  {
  byte descripteur_format, vitess;

  /* Prparation du code Init. */
  switch (longueur)
    {
    case '5': {descripteur_format = 0; break;}
    case '6': {descripteur_format = 1; break;}
    case '7': {descripteur_format = 2; break;}
    case '8': {descripteur_format = 3; break;}
    default : return(TRUE);
    }
  switch (stop_bit)
    {
    case '1': break;
    case '2': {descripteur_format = descripteur_format + 0x04; break;}
    default : return(TRUE);
    }
  switch (parity)
    {
    case 'n':
    case 'N': break;
    case 'i':
    case 'I': {descripteur_format = descripteur_format + 8; break;}
    case 'p':
    case 'P': {descripteur_format = descripteur_format + 24; break;}
    case 't':
    case 'T': {descripteur_format = descripteur_format + 40; break;}
    case 'r':
    case 'R': {descripteur_format = descripteur_format + 56; break;}
    default : return(TRUE);
    }
  switch (vitesse)
    {
    case   110: {vitess = 0; break;}
    case   150: {vitess = 1; break;}
    case   300: {vitess = 2; break;}
    case   600: {vitess = 3; break;}
    case  1200: {vitess = 4; break;}
    case  2400: {vitess = 5; break;}
    case  4800: {vitess = 6; break;}
    case  9600: {vitess = 7; break;}
    case 19200: {vitess = 8; break;}
    case 28800: {vitess = 9; break;}
    case 38400: {vitess = 10; break;}
    case 57600: {vitess = 11; break;}
    case 115200: {vitess = 12; break;}
    default : return(TRUE);
    }
  inregs.h.bl = descripteur_format;
  inregs.h.bh = vitess;
  inregs.x.dx = CommPort;
  inregs.h.ah = 0x0B;
  /* Int. d'init. */
  if (RS_accessible)
    {
    int86(0x14,&inregs,&outregs);
    *Taille_InBuff = outregs.x.bx;
    *Taille_OutBuff = outregs.x.cx;
    return(FALSE);
    }
  else return(UN_CHECKED);
  }


byte Init_status()
/* Cette fonction retourne, pour le N du port "CommPort", le format de comm.*/
/* La variable "Format" est mise  jour en retour de fonction.               */
/* UN_CHECKED est retourn si la prsence du pilote n'a pas encore t vri- */
/* -fie, TRUE dans le cas contraire.                                        */
  {
  static byte buff[3];
  /* */
  if (RS_accessible)
    {
    /* Demande si prsence d'erreur(s). */
    segregs.es = FP_SEG(&buff[0]);     /* 1ire adresse de rangement...      */
    inregs.x.di = FP_OFF(&buff[0]);
    inregs.x.dx = CommPort;
    inregs.h.ah = 0x0C;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour des indicateurs  l'appelant. */
    Format = (buff[1]*256) + buff[2];
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Reset_Init_status(word Format)
/* R-active le format indiqu "Format" sue le port "CommPort".              */
/* Retourne UN_CHECKED si la prsence du pilote en RAM non vrifie. Sinon,  */
/* retourne 1 si paramtres incorrectes (erreur=TRUE), 0 (FALSE) si OK.      */
  {
  if (RS_accessible)
    {
    inregs.x.bx = Format;
    inregs.x.dx = CommPort;
    inregs.h.ah = 0x0B;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte flush_buffers()
/* Vide totalement le buffer d'mission ET le buffer de rception.           */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x0E;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte flush_InBuff()
/* Rinitialise les rceptions et vide totalement le buffer de rception.    */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x0F;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte flush_OutBuff()
/* Rinitialise les missions et vide totalement le buffer d'mission.       */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x10;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte ResetCOM_and_TimMAX(byte RdTim_MAX, byte *Old_RdTim_MAX,
     byte WrTim_MAX, byte *Old_WrTim_MAX)
/* R-initialise la chaine de comm. complte et redfini les valeurs des     */
/* dlais autoriss avant d'obtenir des erreurs de time-out en lecture et en */
/* criture. Les anciennes valeurs sont rendues dans "*Old_RdTim_MAX" et     */
/* "*Old_WrTim_MAX".                                                         */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x11;
    inregs.h.bh = RdTim_MAX;
    inregs.h.bl = WrTim_MAX;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    *Old_RdTim_MAX = outregs.h.bh;
    *Old_WrTim_MAX = outregs.h.bl;
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Status_Trait_Erreurs(byte *mode, byte *code)
/* Indique l'tat de fonctionnement du processus de substitution des octets  */
/* errons en rception.                                                     */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x12;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    *mode = outregs.h.al;
    *code = outregs.h.bl;
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Def_Trait_Erreurs(byte mode, byte code)
/* Contrle du processus de substitution des octets errons en rception.    */
/* Retourne TRUE si "mode" non valide; FALSE sinon; UN_CHECKED si port inacc.*/
  {
  if (RS_accessible)
    {
    inregs.h.al = mode;
    inregs.h.bl = code;
    inregs.x.dx = CommPort;
    inregs.h.ah = 0x13;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte change_com_port(byte Num_port)
/* Change le numro de port srie utilis 0  3 pour COM1:  COM4:...        */
/* Pour STARCOMF: ferme tous les ports sauf celui spcifi.                  */
  {
  if (RS_accessible)
    {
    CommPort = Num_port;
    inregs.x.dx = CommPort;
    inregs.h.ah = 0x08;
    int86(0x14,&inregs,&outregs);
    /* Retour de fonction */
    return(outregs.h.ah);              /* Code d'erreur renvoy.             */
    }
  else return(UN_CHECKED);
  }


byte Attend_Buff_ems_vide(byte Temps_maxi)
/* Cette fonction permet d'attendre que le buffer d'mission du port spcifi*/
/* soit compltement vide. L'attente ne dpassera pas "Temps_maxi"...        */
/* Le code retourn vaut: 0 si le buffer est finalement bien vid,           */
/*                        4 s'il y a eu time-out en attente,                 */
/*                        5 si le port n'existe pas,                         */
/*                        UN_CHECKED si la prsence du pilote n'a pas encore */
/*                        t vrifie.                                      */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x1B;                /* Code fonction.                     */
    inregs.h.al = Temps_maxi;
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    return(outregs.h.ah);              /* Code d'erreur renvoy.             */
    }
  else return(UN_CHECKED);
  }


byte CheckBufferIn(word *Nb_a_lire)
/* Rends FALSE si le BuffIn est vide; Retourne TRUE sinon.                   */
  {
  if (RS_accessible)
    {
    inregs.x.dx = CommPort;
    inregs.h.ah = 0x1C;
    int86(0x14,&inregs,&outregs);
    *Nb_a_lire = outregs.x.cx;
    if ( (outregs.h.ah==0) & (outregs.h.al==0) ) return(TRUE);
      else return(FALSE);
    }
  else return(UN_CHECKED);
  }


byte CheckBufferOut(word *Nb_libre)
/* Rends TRUE si BuffOut n'est pas plein; Retourne FALSE sinon.              */
  {
  if (RS_accessible)
    {
    inregs.x.dx = CommPort;
    inregs.h.ah = 0x1D;
    int86(0x14,&inregs,&outregs);
    *Nb_libre = outregs.x.cx;
    if ( (outregs.h.ah==0) & (outregs.h.al==0) ) return(TRUE);
      else return(FALSE);
    }
  else return(UN_CHECKED);
  }


byte ReadSerie(byte *car, word nombre, word *Nb_received)
/* Cette fonction permet la lecture de "nombre" octets depuis la voie srie. */
/* Le code retourn vaut: 0 si la lecture de(s) octet(s) est apparue correcte*/
/*                        2 si il y a rception d'un break-interrupt,        */
/*                        3 si vous avez demand de lire aucun octet (!),    */
/*                        4 si un time-out est survenu en lecture,           */
/*                        5 si le port n'existe pas,                         */
/*                        UN_CHECKED si la prsence du pilote n'a pas encore */
/*                        t vrifie.                                      */
  {
  if (RS_accessible)
    {
    /* Rception */
    inregs.h.ah = 0x14;                /* Code fonction de lecture.          */
    inregs.x.cx = nombre;              /* nb. d'octet(s)  lire.             */
    segregs.es = FP_SEG(car);          /* Adresse o stocker le(s) octet(s). */
    inregs.x.di = FP_OFF(car);
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour de fonction */
    *Nb_received = outregs.x.cx;
    return(outregs.h.ah);              /* Code d'erreur renvoy.             */
    }
  else return(UN_CHECKED);
  }


byte WriteSerie(byte *car, word nombre, word *Nb_written)
/* Cette fonction permet l'criture d'une chaine de "nombre" octets.         */
/* Le code retourn vaut: 0 si l'criture de(s) octet(s) est apparue correcte*/
/*                        3 si vous avez demand d'envoyer aucun octet (!),  */
/*                        4 si un time-out est survenu en criture,          */
/*                        5 si le port n'existe pas,                         */
/*                        UN_CHECKED si la prsence du pilote n'a pas encore */
/*                        t vrifie.                                      */
  {
  if (RS_accessible)
    {
    /* Envoi des donnes */
    inregs.h.ah = 0x15;                /* Code fonction d'criture.          */
    inregs.x.cx = nombre;              /* nb. d'octet(s)  crire            */
    segregs.es = FP_SEG(car);          /* Adresse du 1ier octet  mettre.   */
    inregs.x.di = FP_OFF(car);
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour de fonction */
    *Nb_written = outregs.x.cx;
    return(outregs.h.ah);              /* Code d'erreur renvoy.             */
    }
  else return(UN_CHECKED);
  }


byte ReadCarSerie(byte *car)
/* Cette fonction permet la lecture sans attente d'un octet depuis le buffer */
/* de rception du port srie spcifi.                                      */
/* Le code retourn vaut: 0 si la lecture de l'octet est apparue correcte,   */
/*                        3 si aucun octet n'est apparu disponible, ou si le */
/*                          port est ferm !                                 */
/*                        5 si le port n'existe pas,                         */
/*                        UN_CHECKED si la prsence du pilote n'a pas encore */
/*                        t vrifie.                                      */
  {
  if (RS_accessible)
    {
    /* Rception */
    inregs.h.ah = 0x16;                /* Code fonction de lecture.          */
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour de fonction */
    *car = outregs.h.al;
    return(outregs.h.ah);              /* Code d'erreur renvoy.             */
    }
  else return(UN_CHECKED);
  }


byte WriteCarSerie(byte car)
/* Cette fonction permet d'crire sans attendre un octet dans le buffer      */
/* d'mission du port srie spcifi.                                        */
/* Le code retourn vaut: 0 si l'criture de l'octet est apparue possible,   */
/*                        3 si l'octet n'a pas t crit (plus de place ou   */
/*                          port ferm !                                     */
/*                        5 si le port n'existe pas,                         */
/*                        UN_CHECKED si la prsence du pilote n'a pas encore */
/*                        t vrifie.                                      */
  {
  if (RS_accessible)
    {
    /* Emission */
    inregs.h.ah = 0x17;                /* Code fonction d'criture.          */
    inregs.h.al = car;
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour de fonction */
    return(outregs.h.ah);              /* Code d'erreur renvoy.             */
    }
  else return(UN_CHECKED);
  }


byte Peek_rcp(byte *car)
/* Cette fonction permet de lire le prochain octet disponible dans le buffer */
/* de rception du port srie spcifi, sans le retirer de ce buffer.        */
/* Le code retourn vaut: 0 si la lecture de l'octet est apparue possible,   */
/*                        3 si aucun octet n'est disponible (ou port ferm), */
/*                        5 si le port n'existe pas,                         */
/*                        UN_CHECKED si la prsence du pilote n'a pas encore */
/*                        t vrifie.                                      */
  {
  if (RS_accessible)
    {
    /* Lecture */
    inregs.h.ah = 0x18;                /* Code fonction de lecture.          */
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour de fonction */
    *car = outregs.h.al;
    return(outregs.h.ah);              /* Code d'erreur renvoy.             */
    }
  else return(UN_CHECKED);
  }


byte Poke_rcp(byte car)
/* Cette fonction permet d'crire (sans attente) un octet dans le buffer de  */
/* rception du port srie spcifi.                                         */
/* Le code retourn vaut: 0 si l'criture de l'octet est apparue possible,   */
/*                        3 si le buffer est plein ou si le port est ferm,  */
/*                        5 si le port n'existe pas,                         */
/*                        UN_CHECKED si la prsence du pilote n'a pas encore */
/*                        t vrifie.                                      */
  {
  if (RS_accessible)
    {
    /* Ecriture */
    inregs.h.ah = 0x19;                /* Code fonction d'criture.          */
    inregs.h.al = car;
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour de fonction */
    return(outregs.h.ah);              /* Code d'erreur renvoy.             */
    }
  else return(UN_CHECKED);
  }


byte WriteCmde(byte *Cmde, byte nombre, byte UseDTR)
/* Cette fonction permet d'envoyer une commande de "nombre" octets  un modem*/
/* Le code retourn vaut: 0 si l'criture de(s) octet(s) est apparue correcte*/
/*                        3 si vous avez demand d'envoyer aucun octet (!),  */
/*                        5 si le port n'existe pas,                         */
/*                        UN_CHECKED si la prsence du pilote n'a pas encore */
/*                        t vrifie.                                      */
  {
  if (RS_accessible)
    {
    /* Envoi des donnes */
    inregs.h.ah = 0x1A;                /* Code fonction d'criture directe.  */
    segregs.es = FP_SEG(Cmde);         /* Adresse du 1ier octet  mettre.   */
    inregs.x.di = FP_OFF(Cmde);
    inregs.h.al = nombre;              /* nb. d'octet(s)  crire            */
    inregs.h.bh = UseDTR;              /* Utiliser DTR comme indic. de cmde ?*/
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour de fonction */
    return(outregs.h.ah);              /* Code d'erreur renvoy.             */
    }
  else return(UN_CHECKED);
  }


byte etat_du_modem()
/* Cette fonction indique l'tat du modem: prt, CTS, porteuse, sonnerie.    */
/* Retourne TRUE s'il y a eu un changement d'tat modem depuis le dernier    */
/* appel; FALSE dans le cas contraire; et UN_CHECKED si pilote non vrifi.  */
/* Voie_active retourne le code ASCII '1' pour la COM1:,'2' pour la COM2:,...*/
/* Les autres variables prennent la valeur VRAI (code 1) ou FAUX (code 0).   */
  {
  static byte buff[10];
  /* */
  if (RS_accessible)
    {
    /* Demande de l'tat du modem. */
    inregs.h.ah = 0x20;
    inregs.h.al = 0;
    segregs.es = FP_SEG(&buff[0]);     /* 1ire adresse de rangement...      */
    inregs.x.di = FP_OFF(&buff[0]);
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour des indicateurs  l'appelant. */
    Voie_active = buff[0] + 0x30 + 1;
    if (buff[1]==1) Pret = TRUE;
      else Pret = FALSE;
    if (buff[2]==1) Clear_to_send = TRUE;
      else Clear_to_send = FALSE;
    if (buff[3]==1) Sonnerie = TRUE;
      else Sonnerie = FALSE;
    if (buff[4]==1) Porteuse = TRUE;
      else Porteuse = FALSE;
    if (buff[6]==1) dDSR = TRUE;
      else dDSR = FALSE;
    if (buff[7]==1) dCTS = TRUE;
      else dCTS = FALSE;
    if (buff[8]==1) dRI = TRUE;
      else dRI = FALSE;
    if (buff[9]==1) dDCD = TRUE;
      else dDCD = FALSE;
    if (buff[5]==1) return(TRUE);
      else return(FALSE);
    }
  else return(UN_CHECKED);
  }


byte set_DTR()
/* Cette fonction positionne  1 le signal DTR du port srie utilis.        */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x20;
    inregs.h.al = 1;
    inregs.x.di = 0x01;
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte clear_DTR()
/* Cette fonction positionne  0 le signal DTR du port srie utilis.        */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x20;
    inregs.h.al = 2;
    inregs.x.di = 0xFE;
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte set_RTS()
/* Cette fonction positionne  1 le signal RTS du port srie utilis.        */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x20;
    inregs.h.al = 1;
    inregs.x.di = 0x02;
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte clear_RTS()
/* Cette fonction positionne  0 le signal RTS du port srie utilis.        */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x20;
    inregs.h.al = 2;
    inregs.x.di = 0xFD;
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte set_OUT1()
/* Cette fonction positionne  1 le signal OUT1 du port srie utilis.       */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x20;
    inregs.h.al = 1;
    inregs.x.di = 0x04;
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte clear_OUT1()
/* Cette fonction positionne  0 le signal OUT1 du port srie utilis.       */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x20;
    inregs.h.al = 2;
    inregs.x.di = 0xFB;
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Send_break(byte duree)
/* Envoit un signal de break-interrupt via le port srie.                    */
  {
  if (RS_accessible)
    {
    inregs.h.bl = duree;
    inregs.h.ah = 0x1E;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Errors_Report()
/* Cette fonction indique si des erreurs ont eu lieu au niveau interne des   */
/* UART durant les transmissions. Si c'est le cas, ces erreurs sont indiques*/
/* en type et en nombre dans les variables globales:                         */
/*           "Engorgement", "Parite", "Stop_bit" et "Break_it"               */
/* Retourne TRUE si au moins une erreur a eu lieu depuis le dernier appel   */
/* cette fonction; FALSE dans le cas contraire; et UN_CHECKED si prsence du */
/* pilote en RAM non vrifie.                                               */
  {
  static byte buff[5];
  /* */
  if (RS_accessible)
    {
    /* Demande si prsence d'erreur(s). */
    inregs.h.ah = 0x1F;
    segregs.es = FP_SEG(&buff[0]);     /* 1ire adresse de rangement...      */
    inregs.x.di = FP_OFF(&buff[0]);
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour des indicateurs  l'appelant. */
    Buff_overflow = buff[0];
    Engorgement = buff[1];
    Parite = buff[2];
    Stop_bit = buff[3];
    Break_it = buff[4];
    return(outregs.h.al);
    }
  else return(UN_CHECKED);
  }


byte Port_free()
/* Retourne l'tat d'accessibilit du port srie en gestion courante. Le     */
/* drapeau de section critique rendu vaut TRUE si on peut accder  la COM:, */
/* FALSE dans le cas contraire. UN_CHECKED est retourn si la prsence du    */
/* pilote en RAM n'a pas t vrifie.                                       */
  {
  if (RS_accessible)
    {
    inregs.x.dx = CommPort;
    inregs.h.ah = 0x25;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.al);
    }
  else return(UN_CHECKED);
  }


byte HandShake_Status(byte Proto_type, byte *SendEnabled)
/* Retourne l'tat d'activit (utilisation et tat) d'1 des proto. matriels.*/
/* S'il est utilis, le code TRUE est retourn. FALSE est rendu              */
/* dans le cas contraire. UN_CHECKED est retourn si pilote non vrifi.     */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x21;
    inregs.h.al = Proto_type;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    *SendEnabled = (byte)outregs.x.di;
    return(outregs.h.al);
    }
  else return(UN_CHECKED);
  }


byte HandShake_Setup(byte Proto_type, byte state)
/* Indique au pilote le mode d'utilisation des protocoles RTS/CTS et DTR/DSR */
/*  respecter.                                                              */
/* UN_CHECKED est retourn si la prsence du pilote n'a pas encore t vri- */
/* -fie, TRUE dans le cas contraire.                                        */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x22;
    inregs.h.al = Proto_type;
    inregs.h.bh = state;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte XonoffShaking_Status(byte *SendEnabled, byte *Nb_Xoff)
/* Retourne l'tat d'activit (utilisation et tat) du protocole Xon/Xoff.   */
/* Si Xon/Xoff est utilis, le code TRUE est retourn. FALSE est rendu       */
/* dans le cas contraire.                                                    */
/* Indique les codes utiliss pour Xon et Xoff, et prcise le nombre de Xoff */
/* reus et non acquits par des Xon conscutis...                           */
/* UN_CHECKED est retourn si la prsence du pilote n'a pas encore t vri- */
/* -fie.                                                                    */
  {
  static byte buff[5];
  /* */
  if (RS_accessible)
    {
    /* Demande le rapport au pilote. */
    inregs.h.ah = 0x23;
    segregs.es = FP_SEG(&buff[0]);     /* 1ire adresse de rangement...      */
    inregs.x.di = FP_OFF(&buff[0]);
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    /* Retour des indicateurs  l'appelant. */
    Xon = buff[0];
    Xoff = buff[1];
    *Nb_Xoff = buff[2];
    *SendEnabled = buff[3];
    return(buff[4]);
    }
  else return(UN_CHECKED);
  }


byte XonoffShaking_Setup(byte state)
/* Indique au pilote le mode d'utilisation du hand-shak. Xon/Xoff et dfinit */
/* les codes  utiliser pour Xon et Xoff (Variables globales du mme nom...) */
/* UN_CHECKED est retourn si la prsence du pilote n'a pas encore t vri- */
/* -fie, TRUE dans le cas contraire.                                        */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x24;
    inregs.h.al = state;
    inregs.h.bh = Xon;
    inregs.h.bl = Xoff;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Set_routine(word Segprog, word Ofsprog)
/* Demande au pilote d'activer la routine indique  chaque it. gnre.     */
/* UN_CHECKED est retourn si la prsence du pilote n'a pas encore t vri- */
/* -fie, TRUE dans le cas contraire.                                        */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x26;
    segregs.es = Segprog;
    inregs.x.di = Ofsprog;
    inregs.x.dx = CommPort;
    int86x(0x14,&inregs,&outregs,&segregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Unset_routine()
/* Demande au pilote de dsactiver la routine indique plus tt.             */
/* UN_CHECKED est retourn si la prsence du pilote n'a pas encore t vri- */
/* -fie, TRUE dans le cas contraire.                                        */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x27;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Set_err_routine(word Segprog, word Ofsprog)
/* Demande au pilote d'activer la routine indique  chaque erreur de traite-*/
/* ment qui apparait sous l'interruption 14h.				     */
/* UN_CHECKED est retourn si la prsence du pilote n'a pas encore t vri- */
/* -fie, TRUE dans le cas contraire.                                        */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x28;
    segregs.es = Segprog;
    inregs.x.di = Ofsprog;
    int86x(0x14,&inregs,&outregs,&segregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Unset_err_routine()
/* Demande au pilote de dsactiver la routine de traitement des erreurs.     */
/* UN_CHECKED est retourn si la prsence du pilote n'a pas encore t vri- */
/* -fie, TRUE dans le cas contraire.                                        */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x29;
    int86(0x14,&inregs,&outregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Verouille()
/* Interdit au pilote d'tre dsinstall depuis la ligne de commande du DOS. */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x2A;
    inregs.h.al = 1;
    inregs.h.bl = 1;
    int86(0x14,&inregs,&outregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte Deverouille()
/* Autorise au pilote d'tre dsinstall depuis la ligne de commande du DOS. */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x2A;
    inregs.h.al = 1;
    inregs.h.bl = 0;
    int86(0x14,&inregs,&outregs);
    return(TRUE);
    }
  else return(UN_CHECKED);
  }


byte GetDeviceName()
/* Retourne le nom de STARCOMM vu comme un DEVICE DRIVER.                    */
/* "NomDevice" est directement mis  jour, sauf erreur 13 !                  */
  {
  char Nom[9];

  strcpy(Nom,"        ");
  if (RS_accessible)
    {
    inregs.h.ah = 0x2B;
    segregs.es = FP_SEG(&Nom[0]);
    inregs.x.di = FP_OFF(&Nom[0]);
    int86(0x14,&inregs,&outregs);
    if (outregs.h.ah==0) strcpy(NomDevice,Nom);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte GetPortDevice()
/* Indique le port srie physique interfac par le DEVICE.                   */
/* Retourne 13 si STARCOMM.EXE n'est pas disponible en mode DEVICE DRIVER.   */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x2C;
    inregs.h.al = 0;
    int86(0x14,&inregs,&outregs);
    if (outregs.h.ah==0) return(outregs.h.al); else return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


byte ResetPortDevice()
/* Oriente COMM: (s'il a t install) sur le port srie "CommPort".         */
/* Rend 0 si port accept; 5 sinon.                                          */
  {
  if (RS_accessible)
    {
    inregs.h.ah = 0x2C;
    inregs.h.al = 1;
    inregs.x.dx = CommPort;
    int86(0x14,&inregs,&outregs);
    return(outregs.h.ah);
    }
  else return(UN_CHECKED);
  }


/* ================== PROCEDURES POUR LE MODE DEVICE DRIVER ================ */
word Open_COMM()
/* Retourne 0 si pas d'erreur; sinon, rend le code DOS de l'erreur...        */
  {
  word error;
  /* */
  if (! port_ouvert)
    {
    inregs.h.ah = 0x3D;
    segregs.ds = FP_SEG(NomDevice);
    inregs.x.dx = FP_OFF(NomDevice);
    inregs.h.al = 2;   /* Ouverture en Lecture/Ecriture */
    intdosx(&inregs,&outregs,&segregs);
    if (outregs.x.cflag != 0)
      {
      port_ouvert = FALSE;
      error = outregs.x.ax;  /* Code de l'erreur */
      }
    else
      {
      handler_voie_serie = outregs.x.ax;
      port_ouvert = TRUE;
      error = 0;
      }
    }
  else error = 0;
  /* Retour du code d'erreur... */
  return(error);
  }


word Close_COMM()
/* Retourne 0 si pas d'erreur; sinon, rend le code DOS de l'erreur...        */
  {
  if (port_ouvert)
    {
    inregs.h.ah = 0x3E;
    inregs.x.bx = handler_voie_serie;
    intdos(&inregs,&outregs);
    if (outregs.x.cflag != 0) return(outregs.x.ax);   /* Code de l'erreur */
      else
        {
        port_ouvert = FALSE;
        return(0);
        }
    }
  else return(0);
  }


byte IOStream_READ(byte *RTS_type, byte *DTR_type, byte *Xonoff_type)
/* Rend UN_OPENED si pilote non ouvert; sinon, rend FALSE aprs avoir mis   */
/* jour les paramtres "Voie_active" ('1'  '4'), "Format", "Xon", "Xoff",   */
/* et ceux de la procdure. Rend TRUE (= "Il y a eu une erreur !") si le     */
/* retour de fonction du DOS  lieu sur un repport de Carry...               */
  {
  static byte buff[8];
  /* */
  if (port_ouvert)
    {
    inregs.h.ah = 0x44;
    inregs.h.al = 2;
    inregs.x.cx = 8;
    segregs.ds = FP_SEG(&buff[0]);
    inregs.x.dx = FP_OFF(&buff[0]);
    inregs.x.bx = handler_voie_serie;
    intdosx(&inregs,&outregs,&segregs);
    if (outregs.x.cflag==0)
      {
      Voie_active = buff[0] + 0x30 + 1;
      Format = (buff[1] * 256) + buff[2];
      *RTS_type = buff[3];
      *DTR_type = buff[4];
      *Xonoff_type = buff[5];
      Xon = buff[6];
      Xoff = buff[7];
      return(FALSE);
      }
    else return(TRUE);
    }
  else return(UN_OPENED);
  }


byte IOStream_WRITE(byte RTS_type, byte DTR_type, byte Xonoff_type)
/* Rend UN_OPENED si pilote non ouvert; sinon, rend FALSE si les paramtres  */
/* taient correctes (pas d'erreur), FALSE sinon (i.e. port srie inconnu...)*/
  {
  static byte buff[8];
  /* */
  if (port_ouvert)
    {
    buff[0] = Voie_active - 0x30 - 1;
    inregs.x.ax = Format;
    buff[1] = inregs.h.ah;
    buff[2] = inregs.h.al;
    buff[3] = RTS_type;
    buff[4] = DTR_type;
    buff[5] = Xonoff_type;
    buff[6] = Xon;
    buff[7] = Xoff;
    inregs.h.ah = 0x44;
    inregs.h.al = 3;
    inregs.x.cx = 8;
    segregs.ds = FP_SEG(&buff[0]);
    inregs.x.dx = FP_OFF(&buff[0]);
    inregs.x.bx = handler_voie_serie;
    intdosx(&inregs,&outregs,&segregs);
    if (outregs.x.cflag==0) return(FALSE);
      else return(TRUE);
    }
  else return(UN_OPENED);
  }


byte CheckCOMMIn()
/* Rends FALSE si le BuffIn est vide; Retourne TRUE sinon.                   */
  {
  if (port_ouvert)
    {
    inregs.h.ah = 0x44;
    inregs.h.al = 6;
    inregs.x.bx = handler_voie_serie;
    intdos(&inregs,&outregs);
    if (outregs.h.al==0) return(FALSE);
      else return(TRUE);
    }
  else return(UN_OPENED);
  }


byte CheckCOMMOut()
/* Indique s'il est possible d'crire des octets dans le buffer d'mission.  */
  {
  if (port_ouvert)
    {
    inregs.h.ah = 0x44;
    inregs.h.al = 7;
    inregs.x.bx = handler_voie_serie;
    intdos(&inregs,&outregs);
    if (outregs.h.al==0) return(TRUE);
      else return(FALSE);
    }
  else return(UN_OPENED);
  }


int ReadCOMM(byte *car, word nombre)
/* Cette fonction permet la lecture de "nombre" octet(s) depuis la voie      */
/* srie qui DOIT avoir t OUVERTE au prhalable.                           */
/* Le code retourn vaut: 0 si la lecture est apparue correcte,              */
/*                        <DosCodeErr> si une erreur s'est produite,         */
/*                        -1 si aucun octet n'a t lu.                      */
  {
  if (port_ouvert)
    {
    /* Rception */
    inregs.h.ah = 0x3F;                  /* Code lecture fichier             */
    inregs.x.bx = handler_voie_serie;    /* Tmoin du fichier srie          */
    inregs.x.cx = nombre;                /* "nombre" octets  lire           */
    segregs.ds = FP_SEG(car);            /* Adrs. pour stocker le(s) octet(s)*/
    inregs.x.dx = FP_OFF(car);
    intdosx(&inregs,&outregs,&segregs);  /* Appel au DOS                     */
    /* Retour de fonction */
    if (outregs.x.cflag != 0) return(outregs.x.ax); /* Code d'erreur         */
      else
        {
        if (outregs.x.ax != 0) return(0); /* Pas d'erreur                    */
          else return(-1);
        }
    }
  else return(UN_OPENED);
  }


int WriteCOMM(byte *car, word nombre)
/* Cette fonction permet l'criture de "nombre" octet(s) vers la voie srie  */
/* qui DOIT avoir t OUVERTE au pralable.                                  */
/* Le code retourn vaut: 0 si l'criture est apparue correcte,              */
/*                        <DosCodeErr> si une erreur s'est produite,         */
/*                        -1 si aucun octet n'a t crit.                   */
  {
  if (port_ouvert)
    {
    /* Envoi des donnes */
    inregs.h.ah = 0x40;                  /* Code d'criture fichier          */
    inregs.x.bx = handler_voie_serie;    /* Tmoin du fichier srie          */
    inregs.x.cx = nombre;                /* nb. d'octet(s)  crire          */
    segregs.ds = FP_SEG(car);            /* Adresse du 1ier octet  emettre  */
    inregs.x.dx = FP_OFF(car);
    intdosx(&inregs,&outregs,&segregs);  /* Appel au DOS                     */
    if (outregs.x.cflag != 0) return(outregs.x.ax);  /* Code d'erreur        */
      else
        {
        if (outregs.x.ax != 0) return(0); /* Pas d'erreur                    */
          else return(-1);
        }
    }
  else return(UN_OPENED);
  }


/* ================ PROCEDURE SPECIALE DE RECHERCE DE STARCOMM============== */
byte Check_STARCOMM_Present()
/* Retourne: 0=pas de pilote, 1=STARCOMM en BIOS, 2=STARCOMM en DEVICE-DRV.  */
/* RS_accessible rends TRUE si le pilote est prsent, FALSE sinon.           */
/* La variable "NomDevice" est mise  jour.                                  */
  {
  static byte check;

  /* Le pilote srie est-il charg en mmoire et accessible par l'INT 14h ? */
  check=0;
  inregs.h.ah=6;
  inregs.h.al=0;
  int86(0x14,&inregs,&outregs);
  if (outregs.h.al!=0)
    {
    RS_accessible = TRUE;
    check=1;
    }
  else RS_accessible = FALSE;
  /* Trouve-t'on l'interface de priphrique en RAM si STARCOMM est dispo ? */
  if (RS_accessible)
    {
    if (GetDeviceName()==0) check=2;
    }
  /* Retour a la procdure appelante. */
  return(check);
  }
